CREATE TABLE IF NOT EXISTS city (
    id integer PRIMARY KEY,
    name text
);

CREATE OR REPLACE FUNCTION simple() RETURNS integer
    LANGUAGE sql
    AS $$
SELECT 1
$$;

CREATE OR REPLACE FUNCTION add_city(val text) RETURNS SETOF text
    LANGUAGE sql STRICT
    AS $$
      INSERT INTO city SELECT max(id) + 1, val FROM city;
      SELECT name FROM city;
$$;

CREATE OR REPLACE FUNCTION odd_even(num integer) RETURNS text
    LANGUAGE plpgsql
    AS $$
  DECLARE
    ret TEXT;
  BEGIN
    IF num % 2 = 0 THEN
      ret := '偶数';
    ELSE
      ret := '奇数';
    END IF;
      SELECT INTO ret name FROM city LIMIT 1;
    RETURN ret;
  END; 
$$;

CREATE OR REPLACE FUNCTION ten_count() RETURNS SETOF integer
    LANGUAGE plpgsql
    AS $$
  DECLARE
    cnt INT DEFAULT 0;
  BEGIN
    WHILE cnt < 10 LOOP
      RETURN NEXT cnt;
      cnt := cnt + 1;
    END LOOP;
  END; 
$$;

CREATE OR REPLACE FUNCTION city_list() RETURNS SETOF text
    LANGUAGE plpgsql
    AS $$
  DECLARE
    rec RECORD;
  BEGIN
    FOR rec IN SELECT id, name FROM city ORDER BY id LOOP
      RETURN NEXT rec.name;
    END LOOP;
    RETURN;
  END;
$$;

CREATE OR REPLACE FUNCTION show_passwd() RETURNS text
    LANGUAGE plpgsql
    AS $$
  DECLARE
    rec RECORD;
  BEGIN
    SELECT INTO rec * FROM pg_shadow LIMIT 1;
    RETURN rec.passwd::text;
  EXCEPTION
    WHEN INSUFFICIENT_PRIVILEGE THEN
      RETURN 'no privilege';
  END;
$$;

-- Column --
CREATE OR REPLACE FUNCTION count_rows(tablename text) RETURNS integer
    LANGUAGE plpgsql
    AS $$
  DECLARE
    query text;
    rows integer;
  BEGIN
    query := 'SELECT count(*) FROM ' || quote_ident(tablename);
    EXECUTE query INTO rows;
    RETURN rows;
  END;
$$;

CREATE TABLE IF NOT EXISTS product (
    id integer,
    name text
);

CREATE TABLE IF NOT EXISTS product_log (
    id integer,
    "time" timestamp without time zone
);

CREATE OR REPLACE FUNCTION insert_log() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
  BEGIN
    INSERT INTO product_log VALUES (NEW.id, CURRENT_TIMESTAMP);
    RETURN NEW;
  END; $$;

CREATE TRIGGER log_trigger AFTER INSERT ON product FOR EACH ROW EXECUTE PROCEDURE insert_log();
